//
//  SystemSetting.swift
//  AsReaderGUN
//
//  Created by mac on 2018/7/16.
//  Copyright © 2018年 asterisk. All rights reserved.
//

import UIKit

final class SystemSetting: NSObject {
    private let BUZZER = "Buzzer"
    private let VIBRATOR = "Vibrator"
    private let BUZZERTIME = "BuzzerTime"
    private let VIBRATORTIME = "VibratorTime"
    private let INVENTORYTIME = "InventoryTime"
    private let IDLETIME = "IdleTime"
    private let AUTOOFFTIME = "AutoOffTime"
    private let QVALUE = "QValue"
    private let ALGORITHM = "Algorithm"
    private let MAXQ = "MaxQ"
    private let MINQ = "MinQ"
    private let SLEEPTIME = "SleepTime"
    private let MASKTYPEVALUE = "MaskTypeValue"
    private let LINKPROFILEVALUE = "LinkProfileValue"
    private let OPERATIONTIME = "OperationTime"
    private let POWERGAIN = "PowerGain"
    private let INVENTORYSESSION = "InventorySession"
    private let SESSIONFLAG = "SessionFlag"
    private let USESELECTIONMASK = "UseSelectionMask"
    private let CONTINUOUSMODE = "ContinuousMode"
    private let RSSIMODE = "RssiMode"
    private let PROPBARCODEMODE = "PropBarcodeMode"
    private let EPCMASKMATCHMODE = "EpcMaskMatchMode"
    private let DISPLAYPC = "Displaypc"
    private let LBTITEMDATA = "LbtItemData"
    private let RFIDDATA = "RfidData"
    private let BARCODEDATA = "BarcodeData"
    private let MAPTAGS = "MapTags"
    private let EPCMASKDATA = "EpcMaskData"
    private let SELECTIONEPCMASKDATA = "SelectionEpcMaskData"
    private let BARCODEPARAMDATA = "barcodeParamData"
    private let barcodeRescanTime = "BarcodeRescanTime"
    private let isNotFirtUsed = "IsNotFirtUsed"
    private let defaultLinkKey = "DefaultLinkProfileValue"
    private weak var appDelegate: AppDelegate?
    private var userDefaults: UserDefaults?
    static private var systemSetting: SystemSetting?
    public final class func shared() -> SystemSetting? {
        if systemSetting == nil {
            systemSetting = SystemSetting()
        }
        return systemSetting
    }
    override init() {
        super.init()
        appDelegate = UIApplication.shared.delegate as? AppDelegate
        userDefaults = UserDefaults.standard
    }
    func asReaderGUNSetting() {
        appDelegate?.mReader?.buzzer = BuzzerState(rawValue: BuzzerState.RawValue(getBuzzer()))
        appDelegate?.mReader?.vibrator = VibratorState(rawValue: VibratorState.RawValue(getVibrator()))
        appDelegate?.mReader?.sleepTime = Int32(getSleepTime())
    }
    func rfidSetting() {
        if appDelegate?.mReader != nil {
            appDelegate?.mReader?.inventoryTime = Int32(getInventoryTime())
            appDelegate?.mReader?.idleTime = Int32(getIdleTime())
            appDelegate?.mReader?.autoOffTime = Int32(getAutoOffTime())
            appDelegate?.mReader?.qValue = Int32(getQValue())
            appDelegate?.mReader?.algorithm = AlgorithmType(rawValue: AlgorithmType.RawValue(getAlgorithm()))
            appDelegate?.mReader?.maxQ = Int32(getMaxQ())
            appDelegate?.mReader?.minQ = Int32(getMinQ())
            appDelegate?.mReader?.linkProfileValue = Int32(getLinkProfileValue())
            appDelegate?.mReader?.defaultLinkProfileValue = Int32(getDefaultLinkProfileValue())
            appDelegate?.mReader?.operationTime = Int32(getOperationTime())
            appDelegate?.mReader?.powerGain = Int32(getPowerGain())
            appDelegate?.mReader?.inventorySession = SessionType(rawValue: SessionType.RawValue(getInventorySession()))
            appDelegate?.mReader?.sessionFlag = SessionFlag(rawValue: SessionFlag.RawValue(getSessionFlag()))
            appDelegate?.mReader?.continuousMode = getContinuousMode()
            appDelegate?.mReader?.rssiMode = getRssiMode()
            if let isJapan = appDelegate?.mReader?.regionName().hasPrefix("Japan") {
                if isJapan {
                    var lbtItemArr = getLbtItemData()
                    if lbtItemArr == nil {
                        if let lbtItems  = appDelegate?.mReader?.getLBT {
                            if var lbtArr: [LbtItem] = lbtItems() as? [LbtItem] {
                                var lbtI: LbtItem = LbtItem.init()
                                for lbtItem in lbtArr {
                                    let lbitem = lbtItem
                                    lbitem.frequency = appDelegate?.mReader?.getLBTFrequency(lbitem.mSlot)
                                    if lbitem.frequency == "0 HZ" {
                                        lbtI = lbitem
                                    }
                                }
                                if let index = lbtArr.index(of: lbtI) {
                                    lbtArr.remove(at: index)
                                }
                                lbtItemArr = lbtArr
                                setLbtItemData(lbtItemArr)
                            }
                        }
                    }
                    appDelegate?.mReader?.setLBT(getLbtItemData())
                }
            }
            appDelegate?.mReader?.maskTypeValue = Int32(getMaskTypeValue())
            switch UInt32(getMaskTypeValue()) {
            case MaskType_NO_MASK.rawValue:
                break
            case MaskType_Selection.rawValue:
                var param: AsSelectMaskParam?
                if let epcMaskData = getEpcMaskData() {
                    var selectionArray: [AsSelectMaskParam] = epcMaskData
                    for index in MIN_SELECTION_MASK..<MAX_SELECTION_MASK {
                        if index < Int(selectionArray.count) {
                            param = selectionArray[Int(index)]
                            appDelegate?.mReader?.setSelectionMask(index, with: param)
                        } else {
                            appDelegate?.mReader?.removeSelectionMask(index)
                        }
                    }
                    if selectionArray.isEmpty {
                        appDelegate?.mReader?.useSelectionMask = SelectFlag(rawValue: SelectFlag.RawValue(getUseSelectionMask()))
                    } else {
                        appDelegate?.mReader?.clearSelectionMask()
                    }
                  } else {
                    appDelegate?.mReader?.clearSelectionMask()

                }
            case MaskType_EPC.rawValue:
                appDelegate?.mReader?.clearEpcMask()
                 if let selectionEpcMaskData = getSelectionEpcMaskData() {
                    for mask in selectionEpcMaskData {
                        appDelegate?.mReader?.addEpcMask(mask)
                    }
                }
                if getEpcMaskMatchMode() {
                    appDelegate?.mReader?.epcMaskMatchMode = true
                } else {
                    appDelegate?.mReader?.epcMaskMatchMode = false
                }
            default:
                break
            }
        }
    }

    func barcodeSetting() {
        if appDelegate?.mReader != nil {
            appDelegate?.mReader?.setPropBarcodeMode(true)
            var res = ResultNoError
            if let arr = getBarcodeParamData() {
                for paramName in arr {
                    if let aRes = appDelegate?.mReader?.setBarcodeParam(paramName) {
                        res = aRes
                    }
                    if res != ResultNoError {
                        return
                    }
                }
            }
            appDelegate?.mReader?.setPropBarcodeMode(false)
        }
    }
    func setNotFirstUsed(_ isFirst: Bool) {
        userDefaults?.set(isFirst, forKey: isNotFirtUsed)
        userDefaults?.synchronize()
    }
    func getNotFirstUsed() -> Bool {
        return userDefaults?.bool(forKey: isNotFirtUsed) ?? false
    }
    func setBuzzer(_ buzzrtType: Int) {
        userDefaults?.set(buzzrtType, forKey: BUZZER)
        userDefaults?.synchronize()
    }
    func setVibrator(_ vibratorType: Int) {
        userDefaults?.set(vibratorType, forKey: VIBRATOR)
        userDefaults?.synchronize()
    }
    func setInventoryTime(_ inventoryTime: Int) {
        userDefaults?.set(inventoryTime, forKey: INVENTORYTIME)
        userDefaults?.synchronize()
    }
    func setIdleTime(_ idleTime: Int) {
        userDefaults?.set(idleTime, forKey: IDLETIME)
        userDefaults?.synchronize()
    }
    func setAutoOffTime(_ autoOffTime: Int) {
        userDefaults?.set(autoOffTime, forKey: AUTOOFFTIME)
        userDefaults?.synchronize()
    }
    func setQValue(_ qValue: Int) {
        userDefaults?.set(qValue, forKey: QVALUE)
        userDefaults?.synchronize()
    }
    func setAlgorithm(_ algorithm: Int) {
        userDefaults?.set(algorithm, forKey: ALGORITHM)
        userDefaults?.synchronize()
    }
    func setMaxQ(_ maxQValue: Int) {
        userDefaults?.set(maxQValue, forKey: MAXQ)
        userDefaults?.synchronize()
    }
    func setMinQ(_ minQValue: Int) {
        userDefaults?.set(minQValue, forKey: MINQ)
        userDefaults?.synchronize()
    }
    func setSleepTime(_ sleepTimeValue: Int) {
        userDefaults?.set(sleepTimeValue, forKey: SLEEPTIME)
        userDefaults?.synchronize()
    }
    func setMaskTypeValue(_ maskTypeValue: Int) {
        userDefaults?.set(maskTypeValue, forKey: MASKTYPEVALUE)
        userDefaults?.synchronize()
    }
    func setLinkProfileValue(_ linkProfileValue: Int) {
        userDefaults?.set(linkProfileValue, forKey: LINKPROFILEVALUE)
        userDefaults?.synchronize()
    }
    func setDefaultLinkProfileValue(_ defaultLinkProfileValue: Int) {
        userDefaults?.set(defaultLinkProfileValue, forKey: defaultLinkKey)
        userDefaults?.synchronize()
    }
    func setOperationTime(_ operationTime: Int) {
        userDefaults?.set(operationTime, forKey: OPERATIONTIME)
        userDefaults?.synchronize()
    }
    func setPowerGain(_ powerGain: Int) {
        userDefaults?.set(powerGain, forKey: POWERGAIN)
        userDefaults?.synchronize()
    }
    func setInventorySession(_ inventorySession: Int) {
        userDefaults?.set(inventorySession, forKey: INVENTORYSESSION)
        userDefaults?.synchronize()
    }
    func setSessionFlag(_ sessionFlag: Int) {
        userDefaults?.set(sessionFlag, forKey: SESSIONFLAG)
        userDefaults?.synchronize()
    }
    func setUseSelectionMask(_ useSelectionMask: Int) {
        userDefaults?.set(useSelectionMask, forKey: USESELECTIONMASK)
        userDefaults?.synchronize()
    }
    func setContinuousMode(_ isContinuousMode: Bool) {
        userDefaults?.set(isContinuousMode, forKey: CONTINUOUSMODE)
        userDefaults?.synchronize()
    }
    func setRssiMode(_ isRssiMode: Bool) {
        userDefaults?.set(isRssiMode, forKey: RSSIMODE)
        userDefaults?.synchronize()
    }
    func setPropBarcodeMode(_ isPropBarcodeMode: Bool) {
        userDefaults?.set(isPropBarcodeMode, forKey: PROPBARCODEMODE)
        userDefaults?.synchronize()
    }
    func setEpcMaskMatchMode(_ isEpcMaskMatchMode: Bool) {
        userDefaults?.set(isEpcMaskMatchMode, forKey: EPCMASKMATCHMODE)
        userDefaults?.synchronize()
    }
    func setDisPlayPC(_ isDisplaypc: Bool) {
        userDefaults?.set(isDisplaypc, forKey: DISPLAYPC)
        userDefaults?.synchronize()
    }
    func setLbtItemData(_ lbtItemData: [Any]?) {
        var lbtData: Data?
        if let aData = lbtItemData {
            lbtData = NSKeyedArchiver.archivedData(withRootObject: aData)
        }
        userDefaults?.set(lbtData, forKey: LBTITEMDATA)
        lbtData = nil
        userDefaults?.synchronize()
    }
    func setRfidData(_ rfidData: [Any]?) {
        var data: Data?
        if let aData = rfidData {
            data = NSKeyedArchiver.archivedData(withRootObject: aData)
        }
        userDefaults?.set(data, forKey: RFIDDATA)
        data = nil
        userDefaults?.synchronize()
    }
    func setBarcodeData(_ barcodeData: [Any]?) {
        var data: Data?
        if let aData = barcodeData {
            data = NSKeyedArchiver.archivedData(withRootObject: aData)
        }
        userDefaults?.set(data, forKey: BARCODEDATA)
        data = nil
        userDefaults?.synchronize()
    }
    func setMapTags(_ mapTags: [AnyHashable: Any]?) {
        var data: Data?
        if let aTags = mapTags {
            data = NSKeyedArchiver.archivedData(withRootObject: aTags)
        }
        userDefaults?.set(data, forKey: MAPTAGS)
        data = nil
        userDefaults?.synchronize()
    }
    func setEpcMaskData(_ epcMaskData: [Any]?) {
        var epcData: Data?
        if let aData = epcMaskData {
            epcData = NSKeyedArchiver.archivedData(withRootObject: aData)
        }
        userDefaults?.set(epcData, forKey: EPCMASKDATA)
        epcData = nil
        userDefaults?.synchronize()
    }
    func setSelectionEpcMaskData(_ selectionEpcMaskData: [Any]?) {
        var selectionEpcData: Data?
        if let aData = selectionEpcMaskData {
            selectionEpcData = NSKeyedArchiver.archivedData(withRootObject: aData)
        }
        userDefaults?.set(selectionEpcData, forKey: SELECTIONEPCMASKDATA)
        selectionEpcData = nil
        userDefaults?.synchronize()
    }
    func setBarcodeParamData(_ barcodeParamData: [Any]?) {
        var barcodeData: Data?
        if let aData = barcodeParamData {
            barcodeData = NSKeyedArchiver.archivedData(withRootObject: aData)
        }
        userDefaults?.set(barcodeData, forKey: BARCODEPARAMDATA)
        barcodeData = nil
        userDefaults?.synchronize()
    }
    func getBuzzer() -> Int {
        return userDefaults?.integer(forKey: BUZZER) ?? Int(Buzzer_High.rawValue)
    }
    func getVibrator() -> Int {
        return userDefaults?.integer(forKey: VIBRATOR) ?? Int(Vibrator_On.rawValue)
    }
    func getInventoryTime() -> Int {
        return userDefaults?.integer(forKey: INVENTORYTIME) ?? 400
    }
    func getIdleTime() -> Int {
        return userDefaults?.integer(forKey: IDLETIME) ?? 0
    }
    func getAutoOffTime() -> Int {
        return userDefaults?.integer(forKey: AUTOOFFTIME) ?? 1800
    }
    func getQValue() -> Int {
        return userDefaults?.integer(forKey: QVALUE) ?? 4
    }
    func getAlgorithm() -> Int {
        return userDefaults?.integer(forKey: ALGORITHM) ?? Int(DynamicQ.rawValue)
    }
    func getMaxQ() -> Int {
        return userDefaults?.integer(forKey: MAXQ) ?? 15
    }
    func getMinQ() -> Int {
        return userDefaults?.integer(forKey: MINQ) ?? 0
    }
    func getSleepTime() -> Int {
        return userDefaults?.integer(forKey: SLEEPTIME) ?? 300
    }
    func getMaskTypeValue() -> Int {
        return userDefaults?.integer(forKey: MASKTYPEVALUE) ?? Int(MaskType_NO_MASK.rawValue)
    }
    func getLinkProfileValue() -> Int {
        return userDefaults?.integer(forKey: LINKPROFILEVALUE) ?? 1
    }
    func getDefaultLinkProfileValue() -> Int {
        return userDefaults?.integer(forKey: defaultLinkKey) ?? 1
    }
    func getOperationTime() -> Int {
        return userDefaults?.integer(forKey: OPERATIONTIME) ?? 0
    }
    func getPowerGain() -> Int {
        return userDefaults?.integer(forKey: POWERGAIN) ??  300
    }
    func getInventorySession() -> Int {
        return userDefaults?.integer(forKey: INVENTORYSESSION) ?? Int(Session_S0.rawValue)
    }
    func getSessionFlag() -> Int {
        return userDefaults?.integer(forKey: SESSIONFLAG) ?? Int(SessionFlag_AB.rawValue)
    }
    func getUseSelectionMask() -> Int {
        return userDefaults?.integer(forKey: USESELECTIONMASK) ?? Int(SelectFlag_All.rawValue)
    }
    func getContinuousMode() -> Bool {
        return userDefaults?.bool(forKey: CONTINUOUSMODE) ?? true
    }
    func getRssiMode() -> Bool {
        return userDefaults?.bool(forKey: RSSIMODE) ?? false
    }
    func getPropBarcodeMode() -> Bool {
        return userDefaults?.bool(forKey: PROPBARCODEMODE) ?? true
    }
    func getEpcMaskMatchMode() -> Bool {
        return userDefaults?.bool(forKey: EPCMASKMATCHMODE) ?? false
    }
    func getDisPlayPC() -> Bool {
        return userDefaults?.bool(forKey: DISPLAYPC) ?? true
    }
    func getLbtItemData() -> [LbtItem]? {
        let tokenObject = userDefaults?.object(forKey: LBTITEMDATA) as? Data
        var array: [LbtItem]?
        if let anObject = tokenObject {
            array = NSKeyedUnarchiver.unarchiveObject(with: anObject) as? [LbtItem]
        }
        return array
    }
    func getBarcodeData() -> [AsBarcodeItem]? {
        let data = userDefaults?.object(forKey: BARCODEDATA) as? Data
        var array: [AsBarcodeItem]?
        if let aData = data {
            array = NSKeyedUnarchiver.unarchiveObject(with: aData) as? [AsBarcodeItem]
        }
        return array
    }
    func getEpcMaskData() -> [AsSelectMaskParam]? {
        let tokenObject = userDefaults?.object(forKey: EPCMASKDATA) as? Data
        var array: [AsSelectMaskParam]?
        if let anObject = tokenObject {
            array = NSKeyedUnarchiver.unarchiveObject(with: anObject) as? [AsSelectMaskParam]
        }
        return array
    }
    func getSelectionEpcMaskData() -> [AsSelectMaskEPCParam]? {
        let tokenObject = userDefaults?.object(forKey: SELECTIONEPCMASKDATA) as? Data
        var array: [AsSelectMaskEPCParam]?
        if let anObject = tokenObject {
            array = NSKeyedUnarchiver.unarchiveObject(with: anObject) as? [AsSelectMaskEPCParam]
        }
        return array
    }
    func getBarcodeParamData() -> [AsParamValue]? {
        let tokenObject = userDefaults?.object(forKey: BARCODEPARAMDATA)as? Data
        var array: [AsParamValue]?
        if let anObject = tokenObject {
            array = NSKeyedUnarchiver.unarchiveObject(with: anObject) as? [AsParamValue]
        }
        return array
    }
    func setBuzzerTime(_ buzzerTime: Int) {
        userDefaults?.set(buzzerTime, forKey: BUZZERTIME)
        userDefaults?.synchronize()
    }
    func getBuzzerTime() -> Int {
        return userDefaults?.integer(forKey: BUZZERTIME) ?? 1
    }
    func setVibratorTime(_ vibratorTime: Int) {
        userDefaults?.set(vibratorTime, forKey: VIBRATORTIME)
        userDefaults?.synchronize()
    }
    func getVibratorTime() -> Int {
        return userDefaults?.integer(forKey: VIBRATORTIME) ?? 1
    }
    func setBarcodeReScanTime(_ barcodeReScanTime: Int) {
        userDefaults?.set(barcodeReScanTime, forKey: barcodeRescanTime)
        userDefaults?.synchronize()
    }
    func getBarcodeReScanTime() -> Int {
        return userDefaults?.integer(forKey: barcodeRescanTime) ?? 0
    }
}
