//
//  TagAccessViewController.swift
//  AsReaderGUN
//
//  Created by mac on 2018/7/13.
//  Copyright © 2018年 asterisk. All rights reserved.
//

import UIKit

final class TagAccessViewController: UIViewController {
    @IBOutlet private weak var selectionTagLabel: UILabel!
    @IBOutlet private weak var messageLabel: UILabel!
    @IBOutlet private weak var pickerView: UIPickerView!
    @IBOutlet private weak var killPasswordSwitch: UISwitch!
    @IBOutlet private weak var accessPasswordSwitch: UISwitch!
    @IBOutlet private weak var epcSwitch: UISwitch!
    @IBOutlet private weak var tidSwitch: UISwitch!
    @IBOutlet private weak var userSwitch: UISwitch!
    @IBOutlet private weak var passwordTextField: UITextField!
    @IBOutlet private weak var powerGainTextField: UITextField!
    @IBOutlet private weak var operationTimeTextField: UITextField!
    @IBOutlet private weak var lockBtn: UIButton!
    @IBOutlet private weak var unlockBtn: UIButton!
    @IBOutlet private weak var permalockBtn: UIButton!
    @IBOutlet private weak var killBtn: UIButton!
    @IBOutlet private weak var setAccessPwdBtn: UIButton!
    @IBOutlet private weak var setKillPwdBtn: UIButton!
    @IBOutlet private weak var clearBtn: UIButton!
    @IBOutlet private weak var maskBtn: UIButton!
    @IBOutlet private weak var powerGainPickerTopView: UIView!
    @IBOutlet private var buttonCollection: [UIButton]!
    @IBOutlet private weak var rssiLabel: UILabel!
    @IBOutlet private weak var phaseLabel: UILabel!
    @IBOutlet private weak var reportRssiSwitch: UISwitch!
    @IBOutlet private var reportRssiCollection: [UILabel]!
    private let pickerViewPowerGain = 0
    private weak var appDelegate: AppDelegate?
    private var currentAlertController: UIAlertController?
    private var isActionResult = false
    private var maskType: MaskType?
    private var systemSetting: SystemSetting?
    var maskValue = ""
    private var powerGainPickerData: [String] = []
    override func viewDidLoad() {
        super.viewDidLoad()
        systemSetting = SystemSetting.shared()
    }
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        let center = NotificationCenter.default
        center.addObserver(self,
                           selector: #selector(asReaderGUNDisconnected(_:)),
                           name: NSNotification.Name(asreaderGUNDisconnected),
                           object: nil)
        appDelegate?.mReader?.setDelegate(self)
    }
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        AppDelegate.showGlobalProgressHUD(withTitle: loading)
        appDelegate = UIApplication.shared.delegate as? AppDelegate
        appDelegate?.mReader?.setDelegate(self)
        if appDelegate?.mReader == nil {
            AppDelegate.dismissGlobalHUD()
            return
        }
        initPowerGainPickerData()
        appDelegate?.mReader?.continuousMode = false
        maskType = (appDelegate?.mReader?.maskTypeValue).map { MaskType(rawValue: MaskType.RawValue($0)) }
        if  !maskValue.isEmpty {
            appDelegate?.initMask(maskValue, maskType: MaskType_Selection)
        }
        isActionResult = false
        if let powerGain =  appDelegate?.mReader?.powerGain {
            let nPower = powerGain
            setPowerGain((Int(nPower / 10)))
        }
        if let operationTime = appDelegate?.mReader?.operationTime {
            setOperationTime("\(operationTime)")
        }
        if let isReportRSSI = appDelegate?.mReader?.rssiMode {
            setReportRssiModeEnabled(isReportRSSI)
        }
        pickerView.reloadAllComponents()
        if  !maskValue.isEmpty {
            selectionTagLabel.text = maskValue
            maskBtn.isEnabled = false
        }
        AppDelegate.dismissGlobalHUD()
    }
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        NotificationCenter.default.removeObserver(self,
                                                  name: NSNotification.Name(asreaderGUNDisconnected),
                                                  object: nil)
        if  !maskValue.isEmpty {
            appDelegate?.exitMask()
        }
    }
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
    @IBAction func passwordBtnTapped(_ sender: UIButton) {
        let alertController = UIAlertController(title: "Password:", message: "Please enter password.", preferredStyle: .alert)
        alertController.addTextField { (textField) in
            textField.keyboardType = .alphabet
            textField.delegate = self
        }
        let cancelAction = UIAlertAction(title: cancel, style: .cancel, handler: nil)
        let okAction = UIAlertAction(title: ok, style: .default, handler: { _ in
            let value = alertController.textFields?.first?.text
            self.setPassword(value)
        })
        alertController.addAction(cancelAction)
        alertController.addAction(okAction)
        present(alertController, animated: true, completion: nil)
    }
    @IBAction func powerGainBtnTapped(_ sender: UIButton) {
        pickerView.tag = pickerViewPowerGain
        pickerView.reloadAllComponents()
        pickerView.isHidden = false
        powerGainPickerTopView.isHidden = false
        if let text = powerGainTextField.text {
            if !text.isEmpty {
                if let row = powerGainPickerData.index(of: text) {
                    pickerView.selectRow(row, inComponent: 0, animated: false)
                    return
                }
            }
        }
        pickerView.selectRow(0, inComponent: 0, animated: false)
    }
    @IBAction func operationTimeBtnTapped(_ sender: UIButton) {
        let alertController = UIAlertController(title: "Operation Time:", message: "Please enter operation time.\n(ex: 30 ms)", preferredStyle: .alert)
        alertController.addTextField { (textField) in
            textField.keyboardType = .numberPad
            textField.delegate = self
        }
        let cancelAction = UIAlertAction(title: cancel, style: .cancel, handler: nil)
        let okAction = UIAlertAction(title: ok, style: .default, handler: { _ in
            var text = alertController.textFields?.first?.text
            guard let value = text else {
                return
            }
            if value.isEmpty {
                text = "0"
            }
            if let aText = text {
                self.setOperationTime(aText)
                self.appDelegate?.mReader?.operationTime = Int32(aText) ?? 0
                self.systemSetting?.setOperationTime(Int(aText) ?? 0)
            }
        })
        alertController.addAction(cancelAction)
        alertController.addAction(okAction)
        present(alertController, animated: true, completion: nil)
    }
    @IBAction func reportRssiSwitchTapped(_ sender: UISwitch) {
        appDelegate?.mReader?.rssiMode = reportRssiSwitch.isOn
        systemSetting?.setRssiMode(reportRssiSwitch.isOn)
        setReportRssiModeEnabled(reportRssiSwitch.isOn)
    }
    @IBAction func lockBtnTapped(_ sender: UIButton) {
        isActionResult = false
        appDelegate?.mReader?.accessPassword = passwordTextField.text
        let param = LockParam()
        param.killPassword = killPasswordSwitch.isOn
        param.accessPassword = accessPasswordSwitch.isOn
        param.epc = epcSwitch.isOn
        param.tid = tidSwitch.isOn
        param.user = userSwitch.isOn
        if let operationTimeText = operationTimeTextField.text {
            let time: Int32 = Int32(operationTimeText) ?? 0
            appDelegate?.mReader?.operationTime = time
            systemSetting?.setOperationTime(Int(time))
        }
        if appDelegate?.mReader?.lock(param) != ResultNoError {
            print("ERROR. Failed to start lock tag")
        }
    }
    @IBAction func unlockBtnTapped(_ sender: UIButton) {
        isActionResult = false
        appDelegate?.mReader?.accessPassword = passwordTextField.text
        let param = LockParam()
        param.killPassword = killPasswordSwitch.isOn
        param.accessPassword = accessPasswordSwitch.isOn
        param.epc = epcSwitch.isOn
        param.tid = tidSwitch.isOn
        param.user = userSwitch.isOn
        if let operationTimeText = operationTimeTextField.text {
            let time: Int32 = Int32(operationTimeText) ?? 0
            appDelegate?.mReader?.operationTime = time
            systemSetting?.setOperationTime(Int(time))
        }
        if appDelegate?.mReader?.unlock(param) != ResultNoError {
        }
    }
    @IBAction func permalockBtnTapped(_ sender: UIButton) {
        isActionResult = false
        appDelegate?.mReader?.accessPassword = passwordTextField.text
        let param = LockParam()
        param.killPassword = killPasswordSwitch.isOn
        param.accessPassword = accessPasswordSwitch.isOn
        param.epc = epcSwitch.isOn
        param.tid = tidSwitch.isOn
        param.user = userSwitch.isOn
        if let operationTimeText = operationTimeTextField.text {
            let time: Int32 = Int32(operationTimeText) ?? 0
            appDelegate?.mReader?.operationTime = time
            systemSetting?.setOperationTime(Int(time))
        }
        if appDelegate?.mReader?.permaLock(param) != ResultNoError {
        }
    }
    @IBAction func killBtnTapped(_ sender: UIButton) {
        isActionResult = false
        let alertController = UIAlertController(title: "Kill Password:", message: "Please enter password.", preferredStyle: .alert)
        alertController.addTextField { (textField) in
            textField.keyboardType = .alphabet
            textField.delegate = self
        }
        let cancelAction = UIAlertAction(title: cancel, style: .cancel, handler: nil)
        let okAction = UIAlertAction(title: ok, style: .default, handler: { _ in
            self.currentAlertController?.dismiss(animated: true, completion: nil)
            self.currentAlertController = UIAlertController(title: "Error",
                                                       message: "Password length is not valid ",
                                                       preferredStyle: .alert)
            let okAction = UIAlertAction(title: cancel, style: .default, handler: nil)
            self.currentAlertController?.addAction(okAction)
            guard let alert = self.currentAlertController else {
                return
            }
            self.present(alert, animated: true, completion: nil)
        })
        alertController.addAction(cancelAction)
        alertController.addAction(okAction)
        present(alertController, animated: true, completion: nil)
    }
    @IBAction func setAccessPwdBtnTapped(_ sender: UIButton) {
        isActionResult = false
        let alertController = UIAlertController(title: "Set Access Password:", message: "Please enter password.", preferredStyle: .alert)
        alertController.addTextField { (textField) in
            textField.keyboardType = .alphabet
            textField.delegate = self
        }
        let cancelAction = UIAlertAction(title: cancel, style: .cancel, handler: nil)
        let okAction = UIAlertAction(title: ok, style: .default, handler: { _ in
            let value = alertController.textFields?.first?.text
            self.appDelegate?.mReader?.accessPassword = self.passwordTextField.text
            self.appDelegate?.mReader?.writeMemory(Bank_Reserved, offset: 2, value: value)
        })
        alertController.addAction(cancelAction)
        alertController.addAction(okAction)
        present(alertController, animated: true, completion: nil)
    }
    @IBAction func setKillPwdBtnTapped(_ sender: UIButton) {
        isActionResult = false
        let alertController = UIAlertController(title: "Set Kill Password:", message: "Please enter password.", preferredStyle: .alert)
        alertController.addTextField { (textField) in
            textField.keyboardType = .alphabet
            textField.delegate = self
        }
        let cancelAction = UIAlertAction(title: cancel, style: .cancel, handler: nil)
        let okAction = UIAlertAction(title: ok, style: .default, handler: { _ in
            let value = alertController.textFields?.first?.text
            self.appDelegate?.mReader?.accessPassword = self.passwordTextField.text
            self.appDelegate?.mReader?.writeMemory(Bank_Reserved, offset: 0, value: value)
        })
        alertController.addAction(cancelAction)
        alertController.addAction(okAction)
        present(alertController, animated: true, completion: nil)
    }
    @IBAction func clearBtnTapped(_ sender: UIButton) {
        selectionTagLabel.text = "Result"
        messageLabel.text = msgTagWait
        rssiLabel.text = "0.0 dB"
        phaseLabel.text = "0.0˚"
    }
    @IBAction func maskBtnTapped(_ sender: UIButton) {
        if maskType == MaskType_Selection {
            performSegue(withIdentifier: "SelectionMask", sender: self)
        } else {
            currentAlertController?.dismiss(animated: true, completion: nil)
            currentAlertController = UIAlertController(title: "",
                                                       message: "Please choosing the Selection Mask type in the RFID Option menu.",
                                                       preferredStyle: .alert)
            let okAction = UIAlertAction(title: ok, style: .default, handler: nil)
            currentAlertController?.addAction(okAction)
            guard let alert = self.currentAlertController else {
                return
            }
            self.present(alert, animated: true, completion: nil)
            return
        }
    }
    @IBAction func pickerCloseBtnTapped(_ sender: UIButton) {
        hidePickerView()
    }
    @IBAction func powerGainPickerOkBtnTapped(_ sender: UIButton) {
        var idx: Int32 = Int32(pickerView.selectedRow(inComponent: 0))
        if let min = appDelegate?.mReader?.powerGainRange().min {
            idx += (min / 10)
        }
        setPowerGain(Int(idx))
        appDelegate?.mReader?.powerGain = idx * 10
        systemSetting?.setPowerGain(Int(idx * 10))
        hidePickerView()
    }
    @IBAction func prepare(forUnwind segue: UIStoryboardSegue) {
    }
}
// MARK: - AsReaderDelegate
extension TagAccessViewController: AsReaderDelegate {
    func on(asReaderTriggerKeyEvent status: Bool) -> Bool {
        if status {
            currentAlertController?.dismiss(animated: true, completion: nil)
            currentAlertController = UIAlertController(title: "Error",
                                                       message: "For this page no Trigger key function.",
                                                       preferredStyle: .alert)
            let okAction = UIAlertAction(title: ok, style: .default, handler: nil)
            currentAlertController?.addAction(okAction)
            guard let alert = self.currentAlertController else {
                return false
            }
            self.present(alert, animated: true, completion: nil)
        }
        return false
    }
    func on(asReaderLeftModeKeyEvent status: Bool) -> Bool {
        guard let rfidModule = appDelegate?.mReader?.isRFIDModule(),
              let barcodeModule = appDelegate?.mReader?.isBarcodeModule() else {
            return false
        }
        return rfidModule && barcodeModule
    }
    func on(asReaderRightModeKeyEvent status: Bool) -> Bool {
        guard let rfidModule = appDelegate?.mReader?.isRFIDModule(),
              let barcodeModule = appDelegate?.mReader?.isBarcodeModule() else {
            return false
        }
        return rfidModule && barcodeModule
    }
    func updateDeviceState(_ error: ResultCode) {
        if error != ResultNoError {
            currentAlertController?.dismiss(animated: true, completion: nil)
            currentAlertController = UIAlertController(title: "Error",
                                                       message: "Failed to get response. Try again.",
                                                       preferredStyle: .alert)
            let okAction = UIAlertAction(title: ok, style: .default, handler: { _ in
                self.navigationController?.popViewController(animated: true)
            })
            currentAlertController?.addAction(okAction)
            guard let alert = self.currentAlertController else {
                return
            }
            self.present(alert, animated: true, completion: nil)
        }
    }
    func changedActionState(_ action: CommandType, resultCode: Int) {
        if action == CommandStop {
            currentAlertController?.dismiss(animated: true, completion: nil)
            if !isActionResult {
                messageLabel.text = msgTagWait
            }
        } else {
            currentAlertController?.dismiss(animated: true, completion: nil)
            currentAlertController = UIAlertController(title: "On progress",
                                                       message: "Waiting Tag Memory",
                                                       preferredStyle: .alert)
            let okAction = UIAlertAction(title: "Stop", style: .default, handler: { _ in
                self.stopAction()
            })
            currentAlertController?.addAction(okAction)
            guard let alert = self.currentAlertController else {
                return
            }
            self.present(alert, animated: true, completion: nil)
        }
    }
    func readTag(_ tag: String?, rssi: Float, phase: Float) {
    }
    func accessResult(_ error: ResultCode, actionState action: CommandType, epc: String?, data: String?, rssi: Float, phase: Float, frequency: Float) {
        isActionResult = true
        currentAlertController?.dismiss(animated: true, completion: nil)
        selectionTagLabel.text = epc
        if error != ResultNoError {
            messageLabel.text = AsResultCode.msg(error)
            currentAlertController = UIAlertController(title: "Error",
                                                       message: AsResultCode.msg(error),
                                                       preferredStyle: .alert)
            let okAction = UIAlertAction(title: ok, style: .default, handler: nil)
            currentAlertController?.addAction(okAction)
            guard let alert = self.currentAlertController else {
                return
            }
            self.present(alert, animated: true, completion: nil)
            return
        }
        messageLabel.text = "Success"
        if !reportRssiSwitch.isOn {
            setReportRssiModeDefault()
        } else {
            rssiLabel.text = String(format: "%.1f dB", rssi)
            phaseLabel.text = String(format: "%.1f˚", phase)
        }
        currentAlertController = UIAlertController(title: "Success",
                                                   message: "Write memory successfully",
                                                   preferredStyle: .alert)
        let okAction = UIAlertAction(title: ok, style: .default, handler: nil)
        currentAlertController?.addAction(okAction)
        guard let alert = self.currentAlertController else {
            return
        }
        self.present(alert, animated: true, completion: nil)
    }
}
// MARK: - UIPickerViewDataSource, UIPickerViewDelegate
extension TagAccessViewController: UIPickerViewDataSource, UIPickerViewDelegate {
    func numberOfComponents(in pickerView: UIPickerView) -> Int {
        return 1
    }
    func pickerView(_ pickerView: UIPickerView, numberOfRowsInComponent component: Int) -> Int {
        if pickerView.tag == pickerViewPowerGain {
            return powerGainPickerData.count
        }
        return 0
    }
    func pickerView(_ pickerView: UIPickerView, titleForRow row: Int, forComponent component: Int) -> String? {
        if pickerView.tag == pickerViewPowerGain {
            return powerGainPickerData[row]
        }
        return "None"
    }
}
// MARK: - UITextFieldDelegate
extension TagAccessViewController: UITextFieldDelegate {
    func textField(_ textField: UITextField, shouldChangeCharactersIn range: NSRange, replacementString string: String) -> Bool {
        let text = textField.text ?? ""
        let length = text.count + string.count - range.length
        if length > maxPasswordLength {
            return false
        }
        if string.isEmpty {
            return true
        }
        if "0123456789ABCDEFabcdef" .contains(string) {
            let newString = (textField.text as NSString?)?.replacingCharacters(in: range, with: string)
            textField.text = newString?.uppercased()
        }
        return false
    }
}
// MARK: - Custom
extension TagAccessViewController {
    func initPowerGainPickerData() {
        var tmpArray: [String] = []
        if let min = appDelegate?.mReader?.powerGainRange().min,
           let max = appDelegate?.mReader?.powerGainRange().max {
            var index = min
            while index <= max {
                tmpArray.append(String(format: "%i.0 dB", index / 10))
                index += 10
            }
        }
        powerGainPickerData = tmpArray
    }
    func hidePickerView() {
        powerGainPickerTopView.isHidden = true
        pickerView.isHidden = true
    }
}
// MARK: - Setters
extension TagAccessViewController {
    func setPassword(_ password: String?) {
        passwordTextField.text = password
    }
    func setPowerGain(_ power: Int) {
        let value = String(format: "%i.0 dB", power)
        if powerGainPickerData.contains(value) {
            if  let index = powerGainPickerData.index(of: value) {
                if powerGainPickerData.count > index {
                    powerGainTextField.text = powerGainPickerData[index]
                }
            }
        }
    }
    func setOperationTime(_ operationTime: String?) {
        operationTimeTextField.text = operationTime
    }
    func stopAction() {
        appDelegate?.mReader?.stop()
    }
    func setReportRssiModeEnabled(_ enabled: Bool) {
        for elem in reportRssiCollection {
            elem.isEnabled = enabled
        }
        if !enabled {
            setReportRssiModeDefault()
        }
        reportRssiSwitch.isOn = enabled
    }
    func setReportRssiModeDefault() {
        rssiLabel.text = String(format: "%.1f dB", 0.0)
        phaseLabel.text = String(format: "%.1f˚", 0.0)
    }
}
// MARK: - NSNotification
extension TagAccessViewController {
    @objc func asReaderGUNDisconnected(_ notification: Notification?) {
        DispatchQueue.main.async {
            self.appDelegate?.mReader = nil
            self.navigationController?.popToRootViewController(animated: true)
        }
    }
}
