//
//  EpcMaskViewController.swift
//  AsReaderGUN
//
//  Created by mac on 2018/7/9.
//  Copyright © 2018年 asterisk. All rights reserved.
//

import UIKit

final class EpcMaskViewController: UIViewController {
    @IBOutlet private weak var tableView: UITableView!
    @IBOutlet private weak var saveBtn: UIButton!
    @IBOutlet private weak var cancelBtn: UIButton!
    @IBOutlet private weak var addBarButtonItem: UIBarButtonItem!
    @IBOutlet private weak var matchModeSwitch: UISwitch!
    private var isEditCell = false
    private weak var appDelegate: AppDelegate?
    private var HUD: MBProgressHUD?
    private var systemSetting: SystemSetting?
    private lazy var maskData: [AsSelectMaskEPCParam] = {
        return []
    }()
    override func viewDidLoad() {
        super.viewDidLoad()
        enableActionWidgets(false)
        systemSetting = SystemSetting.shared()
        AppDelegate.showGlobalProgressHUD(withTitle: loading)
        appDelegate = UIApplication.shared.delegate  as? AppDelegate
        if appDelegate?.mReader == nil {
            AppDelegate.dismissGlobalHUD()
            return
        }
        initLoadMask()
        let matchMode = appDelegate?.mReader?.epcMaskMatchMode ?? false
        matchModeSwitch.isOn = matchMode
        AppDelegate.dismissGlobalHUD()
    }
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        NotificationCenter.default.addObserver(self,
                                               selector: #selector(asReaderGUNDisconnected),
                                               name: NSNotification.Name(rawValue: asreaderGUNDisconnected),
                                               object: nil)
        appDelegate?.mReader?.setDelegate(self)
        isEditCell = false
    }
   override  func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        NotificationCenter.default.removeObserver(self,
                                                  name: NSNotification.Name(asreaderGUNDisconnected),
                                                  object: nil)
    }
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "EpcMaskCell" {
            let epcVC  = segue.destination as? EpcMaskEditViewController
            var indexPath: IndexPath?
            if let aSender = sender as? UITableViewCell {
                indexPath = tableView.indexPath(for: aSender)
            }
            let index = indexPath?.row ?? 0
            epcVC?.selectMaskParam = maskData[index]
            isEditCell = true
        }
    }
    @IBAction func prepare(forUnwind segue: UIStoryboardSegue) {
        let sourceViewController  = segue.source as? EpcMaskEditViewController
        if !isEditCell {
            if let aParam = sourceViewController?.selectMaskParam {
                maskData.append(aParam)
            }
        }
        tableView.reloadData()
    }
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
    @IBAction func saveBtnTapped(_ sender: UIButton) {
        enableActionWidgets(false)
        AppDelegate.showGlobalProgressHUD(withTitle: "Saving...")
        appDelegate?.mReader?.clearEpcMask()
        for mask in maskData {
            appDelegate?.mReader?.addEpcMask(mask)
        }
        if matchModeSwitch.isOn {
            appDelegate?.mReader?.epcMaskMatchMode = true
            systemSetting?.setEpcMaskMatchMode(true)
        } else {
            appDelegate?.mReader?.epcMaskMatchMode = false
            systemSetting?.setEpcMaskMatchMode(false)
        }
        systemSetting?.setSelectionEpcMaskData(maskData)
        enableActionWidgets(true)
        AppDelegate.dismissGlobalHUD()
        let alertController = UIAlertController(title: "Success",
                                                message: "Saved successfully",
                                                preferredStyle: .alert)
        let okAction = UIAlertAction(title: ok, style: .default, handler: { _ in
            self.navigationController?.popViewController(animated: true)
        })
        alertController.addAction(okAction)
        present(alertController, animated: true, completion: nil)
    }
}
// MARK: - UITableViewDelegate, UITableViewDataSource
extension EpcMaskViewController: UITableViewDataSource, UITableViewDelegate {
    func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return maskData.count
    }
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "EpcMaskCell", for: indexPath) as? EpcMaskTableViewCell
        let maskParam = maskData[indexPath.row]
        cell?.setOffsetValue("\(maskParam.offset) bit")
        cell?.setLengthValue("\(maskParam.length) bit")
        cell?.setMaskValue(maskParam.mask)
        guard let aCell = cell else {
            return UITableViewCell()
        }
        return aCell
    }
    func tableView(_ tableView: UITableView, editingStyleForRowAt indexPath: IndexPath) -> UITableViewCellEditingStyle {
        return .delete
    }
    func tableView(_ tableView: UITableView, commit editingStyle: UITableViewCellEditingStyle, forRowAt indexPath: IndexPath) {
        maskData.remove(at: indexPath.row)
        tableView.reloadData()
    }
}
extension EpcMaskViewController: AsReaderDelegate {
    func on(asReaderTriggerKeyEvent status: Bool) -> Bool {
        return false
    }
    func on(asReaderLeftModeKeyEvent status: Bool) -> Bool {
        guard let rfidModule = appDelegate?.mReader?.isRFIDModule(),
              let barcodeModule = appDelegate?.mReader?.isBarcodeModule() else {
            return false
        }
        return rfidModule && barcodeModule
    }
    func on(asReaderRightModeKeyEvent status: Bool) -> Bool {
        guard let rfidModule = appDelegate?.mReader?.isRFIDModule(),
              let barcodeModule = appDelegate?.mReader?.isBarcodeModule() else {
            return false
        }
        return rfidModule && barcodeModule
    }
    func updateDeviceState(_ error: ResultCode) {
    }
}
// MARK: - Custom
extension EpcMaskViewController {
    func initLoadMask() {
        var param: AsSelectMaskEPCParam?
        if let count = appDelegate?.mReader?.epcMaskCount() {
            for index in 0..<count {
                param = appDelegate?.mReader?.getEpcMask(Int32(index))
                if let aParam = param {
                    maskData.append(aParam)
                }
            }
            tableView.reloadData()
            enableActionWidgets(true)
        }
    }
    func enableActionWidgets(_ enabled: Bool) {
        let count = Int(maskData.count)
        addBarButtonItem.isEnabled = enabled && count < MAX_SELECTION_MASK
        saveBtn.isEnabled = enabled
        cancelBtn.isEnabled = enabled
    }
}
// MARK: - NSNotification
extension  EpcMaskViewController {
    @objc func asReaderGUNDisconnected(_ notification: Notification?) {
        DispatchQueue.main.async {
            self.appDelegate?.mReader = nil
            self.navigationController?.popToRootViewController(animated: true)
        }
    }
}
